#!/bin/bash

if ! declare -f Echo > /dev/null ; then
  Echo() {
    echo "$@"
  }
fi

NEWROOT=${NEWROOT:-/mnt}
export NEWROOT

if [ -e /usr/bin/plymouth ] ; then
    mkfifo /progress
    bash -c 'while true ; do read msg < /progress ; plymouth message --text="$msg" ; done ' &
    PROGRESS_PID=$!
fi

rm /etc/machine-id
rm /var/lib/dbus/machine-id
dbus-uuidgen --ensure
systemd-machine-id-setup

salt_device=${salt_device:-$root}

mkdir -p $NEWROOT
if [ -n "$salt_device" ] && mount "$salt_device" $NEWROOT ; then
    for sd in $NEWROOT/etc/salt $NEWROOT/salt $NEWROOT ; do
        if [ -f $sd/minion_id ] ; then # find valid salt configuration
            mkdir -p /etc/salt
            cp -pr $sd/* /etc/salt
            # remove activation key grain copied from the disk with the rest of configuration
            rm -f /etc/salt/minion.d/kiwi_activation_key.conf
            HAVE_MINION_ID=y
            break
        fi
    done
    umount $NEWROOT
fi

mkdir -p /etc/salt/minion.d
cat > /etc/salt/minion.d/grains-initrd.conf <<EOT
grains:
  saltboot_initrd: True
EOT

MACHINE_ID=`salt-call --local --out newline_values_only grains.get machine_id`

# store machine id in grains permanently so it does not change when we switch to
# initrd and back
# this is not needed for SALT but SUSE Manager seems to rely on it
cat > /etc/salt/minion.d/grains-machine_id.conf <<EOT
grains:
  machine_id: $MACHINE_ID
EOT

echo $MACHINE_ID > /etc/machine-id

if [ -z "$HAVE_MINION_ID" ] ; then
    SMBIOS_MANUFACTURER=`salt-call --local --out newline_values_only smbios.get system-manufacturer | tr -d -c 'A-Za-z0-9_-'`
    SMBIOS_PRODUCT=`salt-call --local --out newline_values_only smbios.get system-product-name | tr -d -c 'A-Za-z0-9_-'`
    SMBIOS_SERIAL=`salt-call --local --out newline_values_only smbios.get system-serial-number | tr -d -c 'A-Za-z0-9_-'`
    # MINION_ID_PREFIX can be specified on kernel cmdline
    if [ -n "$MINION_ID_PREFIX" ] ; then
        echo "$MINION_ID_PREFIX.$SMBIOS_MANUFACTURER-$SMBIOS_PRODUCT-$SMBIOS_SERIAL-$MACHINE_ID" > /etc/salt/minion_id
    else
        echo "$SMBIOS_MANUFACTURER-$SMBIOS_PRODUCT-$SMBIOS_SERIAL-$MACHINE_ID" > /etc/salt/minion_id
    fi
    cat > /etc/salt/minion.d/grains-minion_id_prefix.conf <<EOT
grains:
  minion_id_prefix: $MINION_ID_PREFIX
EOT
fi

CUR_MASTER=`salt-call --local --out newline_values_only grains.get master`
# do we have master explicitly configured?
if [ -z "$CUR_MASTER" -o "salt" == "$CUR_MASTER" ] ; then
    # either we have MASTER set on commandline
    # or we try to resolve the 'salt' alias
    if [ -z "$MASTER" ] ; then
        MASTER=`dig +short -t CNAME salt.$DOMAIN | sed -e 's|\.$||' `
    fi
    # if it fails, do nothing and let it fall back to 'salt'
    if [ -n "$MASTER" ] ; then
        echo "master: $MASTER" > /etc/salt/minion.d/master.conf
    fi
    # else
    # ... if it fails, do nothing and let it fall back to 'salt'
fi

if [ -z "$kiwidebug" ];then
    salt-minion -d
else
    salt-minion -d --log-file-level all
fi

sleep 1

SALT_PID=`cat /var/run/salt-minion.pid`

if [ -z "$SALT_PID" ] ; then
   systemException \
       "SALT Minion did not start" \
       "reboot"
fi

MINION_ID="`salt-call --local --out newline_values_only grains.get id`"
MINION_FINGERPRINT="`salt-call --local --out newline_values_only key.finger`"

Echo
Echo "SALT Minion ID:"
Echo "$MINION_ID"
Echo
Echo "SALT Minion key fingerprint:"
Echo "$MINION_FINGERPRINT"
Echo

if [ -e /progress ] ; then
    # split line into two to fit to screen. Need tripple \ to properly pass through
    echo "Terminal ID: $MINION_ID\\\nFingerprint: $MINION_FINGERPRINT" > /progress
fi

while kill -0 "$SALT_PID" >/dev/null 2>&1; do
    sleep 1
done

if [ -f /salt_config ] ; then
  . /salt_config
fi

[ -n "$PROGRESS_PID" ] && kill $PROGRESS_PID

if [ "$systemIntegrity" = "unknown" ] ; then
   systemException \
       "SALT Minion did not create valid configuration" \
       "reboot"
fi

cat > /etc/salt/minion.d/grains-initrd.conf <<EOT
grains:
  saltboot_initrd: False
EOT

cp -pr /etc/salt $NEWROOT/etc
echo $MACHINE_ID > $NEWROOT/etc/machine-id

mkdir -p $NEWROOT/var/log/salt
num=1
while [ -e $NEWROOT/var/log/salt/saltboot_$num ] ; do
  num=$(( num + 1 ))
done
cp -pr /var/log/salt $NEWROOT/var/log/salt/saltboot_$num

if [ -n "$kernelAction" ] ; then
  ####
  # kiwi code to install grub and update initrd for local boot
  activateImage
  chroot $NEWROOT /bin/bash -c \
        "/preinit"
  chroot $NEWROOT /bin/bash -c \
        ". /include ; exec 2>>$ELOG_FILE ; set -x ; cleanImage"
  ####

  umount -a
  sync
  if [ "$kernelAction" = "reboot" ] ; then
    Echo "Reboot with correct kernel version"
    sleep 10
    reboot -f
  elif [ "$kernelAction" = "kexec" ] ; then
    kexec -e
    Echo "Kexec failed, reboot with correct kernel version"
    sleep 10
    reboot -f
  fi
fi
